package se.petterhansson.gp;

public abstract class GPNode implements Cloneable {
	private GPNode parent;
	private GPNode[] child;
	
	public GPNode(int arity) {
		child = new GPNode[arity];
	}
	
	public void setChild(int i, GPNode c) {
		assert (c.parent == null);
		child[i] = c;
		c.parent = this;
	}
	
	public void removeChild(int i) {
		child[i].parent = null;
		child[i] = null;
	}
	
	public GPNode getChild(int i) {
		return child[i];
	}
	
	public int getChildIndex(GPNode c) {
		for (int i = 0; i < child.length; ++i) {
			if (child[i] == c) {
				return i;
			}
		}
		
		return -1;
	}
	
	public int getArity() {
		return child.length;
	}
	
	public GPNode getParent() {
		return parent;
	}
	
	public int getTreeSize() {
		int size = 1;
		for (int i = 0; i < child.length; ++i) {
			if (child[i] != null) {
				size += child[i].getTreeSize();
			}
		}
		
		return size;
	}
	
	public Object clone() throws CloneNotSupportedException {
		GPNode clone = (GPNode) super.clone();
		clone.parent = null; //not necessary but guarantees detection of any bugs
		clone.child = child.clone();
		
		for (int i = 0, n = getArity(); i < n; ++i) {
			GPNode child = getChild(i);
			
			if (child != null) {
				clone.child[i] = (GPNode) child.clone();
				clone.child[i].parent = clone;
			}
		}
		
		return clone;
	}
	
	public abstract String getPrefix();
}
