package se.petterhansson.sevengp;

import se.petterhansson.gp.Individual;
import se.petterhansson.sevengp.gp.SevenRandomSource;
import se.petterhansson.sevengp.gp.alphabet.SevenGPNode;

class Specimen {
	//non-persistent
	public final int sessionId; //should be unique for each execution of the program
	public int score;
	public int games;
	
	//persistent
	public Individual<SevenGPNode> play;
	public Individual<SevenGPNode> send;
	
	public Specimen(int id, SevenRandomSource src) {
		sessionId = id;
		play = new Individual<SevenGPNode>(6, 6, src);
		send = new Individual<SevenGPNode>(6, 6, src);
	}
	
	public Specimen(int id, Specimen parent, SevenRandomSource src) {
		sessionId = id;
		play = new Individual<SevenGPNode>(2, parent.play, src);
		send = new Individual<SevenGPNode>(2, parent.send, src);
	}
	
	public Specimen(int id, Specimen parent1, Specimen parent2) {
		sessionId = id;
		play = new Individual<SevenGPNode>(parent1.play, parent2.play);
		send = new Individual<SevenGPNode>(parent1.send, parent2.send);
	}
	
	public Specimen(int id, Specimen parent, boolean alterMagnitudes) {
		sessionId = id;
		play = new Individual<SevenGPNode>(parent.play);
		send = new Individual<SevenGPNode>(parent.send);
		
		if (alterMagnitudes) {
			play.getRoot().alterMagnitudes();
			send.getRoot().alterMagnitudes();
		}
	}
	
	public float getWinRatio() {
		return (float) score / games;
	}
	
	public float getFitness() {
		int size = play.getTreeSize() + send.getTreeSize();
		int factor = Math.max(0, size - 20);
		return getWinRatio() - factor * 0.0004f;
	}
	
	public String getLuaTable(String indent) {
		StringBuilder str = new StringBuilder();
		
		str.append(indent);
		str.append("{\n");
		str.append(indent);
		str.append("  play = function (env) return " + play.getRoot().getLuaExpr() + " end,\n");
		str.append(indent);
		str.append("  send = function (env) return " + send.getRoot().getLuaExpr() + " end,\n");
		str.append(indent);
		str.append("}");
		
		return str.toString();
	}
}
