package se.petterhansson.sevengp.gp;

import se.petterhansson.gp.Individual;
import se.petterhansson.sevengp.game.SevenGame;
import se.petterhansson.sevengp.game.SevenPlayer;
import se.petterhansson.sevengp.gp.alphabet.SevenGPNode;
import se.petterhansson.sevengp.gp.alphabet.SevenGameGPEnv;

public class SevenGPPlayer extends SevenPlayer {
	private Individual<SevenGPNode> playIndividual;
	private Individual<SevenGPNode> sendIndividual;
	private final SevenGameGPEnv env;
	
	public SevenGPPlayer(SevenGame game, int index, int[] cards,
			Individual<SevenGPNode> play, Individual<SevenGPNode> send) {
		super(game, index, cards);
		env = new SevenGameGPEnv();
		env.game = game;
		
		playIndividual = play;
		sendIndividual = send;
	}

	@Override
	public int playCard() {
		//prepare environment
		env.game = myGame;
		env.player = myIndex;
		
		//check all cards for the best card (according to GP individual)
		float bestScore = Float.NEGATIVE_INFINITY;
		int bestCard = -1;
		int bestIndex = -1;
		
		for (int i = 0, n = cards.size(); i < n; ++i) {
			env.card = cards.get(i);
			
			if (myGame.canCardBePlayed(env.card)) {
				if (bestIndex == -1) {
					bestCard = env.card;
					bestIndex = i;
				}
				
				//check score -> higher score = better chance of selection
				float score = playIndividual.getRoot().getValue(env);
				if (score > bestScore) {
					bestCard = env.card;
					bestIndex = i;
					bestScore = score;
				}
			}
		}
		
		assert (bestCard != -1);
		cards.remove(bestIndex);
		return bestCard;
	}

	@Override
	public void receiveCard(int card) {
		cards.add(card);
	}

	@Override
	public int sendCard() {
		//prepare environment
		env.player = myIndex;
		
		//check all cards for the best card (according to GP individual)
		float bestScore = Float.NEGATIVE_INFINITY;
		int bestCard = -1;
		int bestIndex = 0;
		
		for (int i = 0, n = cards.size(); i < n; ++i) {
			env.card = cards.get(i);
			
			//check score -> higher score = better chance of selection
			float score = sendIndividual.getRoot().getValue(env);
			if (score > bestScore) {
				bestCard = env.card;
				bestIndex = i;
				bestScore = score;
			}
		}
		
		if (bestCard == -1) {
			System.err.println("Warning, no card was selected - algorithm error?");
			bestCard = 0;
		}
		
		cards.remove(bestIndex);
		return bestCard;
	}
}
